#    most of this task is now taken from the standard application

set VERSION "CAEN"

#    put some default environment variables before loading common 


if {[info exists env(VMEServer)]} then {set resource $env(VMEServer)} else {set resource VME}
set VMEserver $resource

if {[info exists env(RDOUTserver)]} then {set resource $env(RDOUTserver)} else {set resource VME}
set RDOUTserver $resource

set LinuxDirectory  [file join $env(MIDASBASE) Data_Acq bin Linux64]
set DAProgram    "CAENExecV1 XXX"
set DADirectory  $LinuxDirectory


loadsource  sys.common

set RDOUThdwre VME

rename click-setup __setup
proc click-setup {arg} {
    if {$arg == 0} {__setup 1} else {__setup $arg}
}

set frametitle "MIDAS Data Acquisition Experiment Control"

set VMEPresent 0
set VMEEnable 0

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set SACPres 0

set TestBuffer ""

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]

lappend actionlist [list "Raw Data Display"  "Dump_Raw_Data"]

lappend registerlist [list NetVar.V1724_TLow     generic  "-n V1724_TLow -t int"]
lappend registerlist [list NetVar.V1724_THigh    generic  "-n V1724_THigh -t int"]
lappend registerlist [list NetVar.V1724_TraceOpt generic  "-n V1724_TraceOpt -t int"]


for {set i 1} {$i <= 24} {incr i} {
    lappend registerlist [list NetVar.DACQ.Stats$i  generic  "-n DACQ_Stats$i -t block -a yes -i 0 -l [expr 64*4*2] -f d"]
    lappend registerlist [list NetVar.DACQ.Rates$i  generic  "-n DACQ_Rates$i -t block -a yes -i 0 -l [expr 64*4*2] -f d"]
}

loadsource sys.DA     ;#  for common data acquisition support

loadsource sys.WWW    ;#   for control via Web services

loadsource sys.2D    ;# for 2D histogrammming feature 

set Supported_Modules [list V1724 V785]

set ProspectusSpectraHandle -1

proc create_spectra {} {
   global env
   global SpectrumServer HistMode EGreport Options
   global 2Dlist 2Dpresent
   upvar #0 ProspectusSpectraHandle f

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set ProspectusSpectraDefs [file join $env(EXPERIMENTDIR) ProspectusSpectraDefs.dat]

   set z [catch {open $ProspectusSpectraDefs w} f]
   if {$z != 0} then {
       midas-warning "Cannot open file $ProspectusSpectraDefs\n$f"
   }

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat 0 0 0 $Options(Stat.shift)" 

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   Create_1D_Histograms

   if {[llength $2Dlist] == 0} {
       if {$2Dpresent == 0} {2D_Histogram_Configuration; close-frame "[self frame-name].2dhistograms"}
   }
   Create_2D_Histograms

   close $f

  execute-command 4 "Mapping spectra" $ProspectusSpectraDefs

   set EGreport $EGreport_SAVE

   clear-footer
}

proc map_spectrum {p} {
   upvar #0 ProspectusSpectraHandle f

   puts $f $p
}


proc Dump_Raw_Data {} {
    global Options

    set Options(Include.S9418) 0  ;# so common code OK for non CAEN modules
    set Options(Include.VF48) 0
    set Options(Include.MDI2) 0

    set Options(Include.VME) 0

    Dump_Raw_VME_Data
}

proc Dump_Raw_VME_Data_VME {} {}

proc Dump_Raw_VME_Data {} {
    global Options Supported_Modules
    global VMEserver

   set info ""

   set CAEN_ModuleNum 0
   foreach m $Supported_Modules {

#       set maxmodule [inform vmecontrol inquire [string tolower $m]count]
#       if {[string length $maxmodule] > 2} {set maxmodule 0}

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/[string tolower $m]#$i.Output_Buffer'  ?object)" } r]
      if {$z!=0} then {break}
      incr maxmodule
   }

       incr CAEN_ModuleNum $maxmodule
   }

   for {set CAEN_ModuleIndex 1} {$CAEN_ModuleIndex <= $CAEN_ModuleNum} {incr CAEN_ModuleIndex} {

      execute-command 19 "Getting Raw Data for CAEN module $CAEN_ModuleIndex" "1 $CAEN_ModuleIndex" 2
      set data [read-register NetVar.EXEC.Response]

      if {[string range [first $data] 0 1] == "0x"} then {set CAEN_ModuleType [expr [first $data]]} else {set CAEN_ModuleType [expr 0x[first $data]]}

      append info "CAEN module $CAEN_ModuleIndex is type V$CAEN_ModuleType\n"
      catch {append info "[Dump_Raw_VME_Data_V$CAEN_ModuleType [lreplace $data 0 0]]\n"}

   }

   if {[info exists Options(Include.VME)] && $Options(Include.VME) == 1} then {append info [Dump_Raw_VME_Data_VME]}

#   midas-information "$info"
    set frame [self frame-name].info

    :frame $frame :label  "Information from [self frame-title]" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $info :font fixed :rows 70 :cols 85
    :show T
}

proc redisplay-txenable {} {
   global  RDOUTEnable txEnable  TSPresent
   if {$TSPresent == 0} then {return}
   if {$RDOUTEnable == 0} then {:modify TXENABLE :value 0 :colours [list red red]; return}
   set txEnable [read-register NetVar.DACQ.Ctxenable]
   :modify TXENABLE :value $txEnable
#   click-special 20
#   set txState [read-register NetVar.EXEC.Parameter]
#   if {$txState == 0} then {
#       :modify TXENABLE :colours [list red red]
#   } else {
#       :modify TXENABLE :colours [list red green]
#   }
    :modify TXENABLE :colours [list grey grey]
}


proc Custom_SetupA  {opt} {

   define_VME_addresses   ;#  setup VME 
}

proc Custom_PreGo {} {
    global Options Supported_Modules

    foreach m $Supported_Modules {[set m]_DAQ-go}
  
#    write-register NetVar.V1724_TLow $Options(V1724_TLow)
#    write-register NetVar.V1724_THigh $Options(V1724_THigh)
    write-register NetVar.V1724_TraceOpt $Options(V1724_TraceOpt)
}

proc Custom_PreStop {} {
    global Supported_Modules

    click-special 23   ;#   pause data readout
    after 500

    foreach m $Supported_Modules {[set m]_DAQ-halt}

    click-special 24   ;#   resume data readout 
    after 1000         ;#    allow to finish

}

set Retry 0
proc click-Retry {v} {global Retry; set Retry $v}

proc check_for_VME {msg} {
    global env Retry

        :frame [self frame-name].vme :label "[self frame-title]" \
           :icon-image "@$env(ICONHOME)/default" \
           :icon-name "[self icon-title]" :resize F

        :panel P :layout T :padx 10 :pady 10
        :layout H; :col-gap 10; :row-gap 10

        :message M  :bold T :justify center :label "$msg"

        :next-row T

        :button CONTINUE :label Continue :w 12 :command "click-Retry 0"

        :show T

        tkwait variable Retry

         destroy-frame [self frame-name].vme
}

proc Custom_PreCheck {opt} {
    global VMEserver

     set z -1
     while {$z != 0} {
         set z [catch {eg "(register :name '/$VMEserver/HasBeenConfigured'  ?value)" } m]
         if {$z!=0 || $m != 1} then {
             check_for_VME "\
The VME server has not been configured\n\
It may recently have been restarted\n\
Start a Web Browser and access http://localhost:8015/VME/Config/VMEConfig.tml\n\
Click on \"Reload\" \"Restore Configuration\" and \"Activate Configuration\"\n\
Then CONTINUE from here\n\
"
             set z -1
         }
     }

     set z -1
     while {$z != 0} {
         set z [catch {eg "(register :name '/$VMEserver/HasBeenRestored'  ?value)" } m]
         if {$z!=0 || $m != 1} then {
             check_for_VME "\
The VME module setup has not been restored\n\
The VME server may recently have been restarted\n\
Start a Web Browser and access http://localhost:8015/VME/V1724/V1724.tml\n\
Click on \"Reload\" \"Restore Settings for all modules\"\n\
Then CONTINUE from here\n\
"
             set z -1
         }
     }

}

proc define_sac_config {} {
   global VMEserver SACPres

#      set maxmodule [inform vmecontrol inquire saccount]
#      if {[string length $maxmodule] > 2} {set maxmodule 0}

   set maxmodule 0
   set z [catch {eg "(register :name '/$VMEserver/sac#1.STR'  ?object)" } m]
   if {$z == 0} then {set maxmodule 1}

   if {$maxmodule != 0} {DefineConfig "0 0 [string range [first $m] 2 end]"}

   set SACPres $maxmodule
}

proc Stop_Electronics {} {
       global SACPres

    if {$SACPres != 0} {write-register sac#1.STR 0}
}

proc Go_Electronics {} {
       global SACPres

    if {$SACPres == 0} {return}     ;#  no SAC is possible

    write-register sac#1.Mode 1   ;#   CAEN mode
    write-register sac#1.MON0 2
    write-register sac#1.STR 9
    write-register sac#1.ARM 0
}

proc Reset_Electronics {} {
       global SACPres
 
    if {$SACPres == 0} {return}     ;#  no SAC is  present

    write-register sac#1.EVCLR 0
}



#    support for Web Service access  -  provides application specific support

proc do_Setup {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Setup:  Acquisition state = $MIDAS_DataAcquisitionCode args = $args"

    after 100 WWW_Setup

   set MIDAS_DataAcquisitionCode 3
}

proc WWW_Setup {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-setup 1} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc do_Reset {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Reset:  Acquisition state = $MIDAS_DataAcquisitionCode args = $args"

    after 100 WWW_Reset

   set MIDAS_DataAcquisitionCode 7
}

proc WWW_Reset {} {
     global MIDAS_DataAcquisitionCode

    if {[da_state] != 0} {set z [catch {click-da 0} m]}

        set MIDAS_DataAcquisitionCode 2
}

proc do_Stop {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Stop:  Acquisition state = $MIDAS_DataAcquisitionCode args = $args"

    after 100 WWW_Stop

   set MIDAS_DataAcquisitionCode 5
}

proc WWW_Stop {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-da 0} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc do_Go {args} {
     global MIDAS_DataAcquisitionCode

#   puts "do_Go:  Acquisition state = $MIDAS_DataAcquisitionCode args = $args"

    after 100 WWW_Go

   set MIDAS_DataAcquisitionCode 4
}

proc WWW_Go {} {
     global MIDAS_DataAcquisitionCode

     set z [catch {click-da 1} m]
     if {$z != 0} {
        set MIDAS_DataAcquisitionCode 6     ;# error
     } else {
        set MIDAS_DataAcquisitionCode [da_state]
     }
}

proc Get_State {args} {
     global MIDAS_DataAcquisitionCode

#   puts "Get_State:  Acquisition state = $MIDAS_DataAcquisitionCode args = $args"

    if {$MIDAS_DataAcquisitionCode == 8} {set MIDAS_DataAcquisitionCode [da_state]}

     return $MIDAS_DataAcquisitionCode
}

proc do_SetOption {state args} {

   if {[llength $args] == 1} {set args [first $args]}

   switch [first $args] {
        ClearStats  {do_clearstats}
        DisableScanTable {execute-command 16 "Disable Scan Table" "2 0 0 0 0 0" 2}
        EnableScanTable  {
            set p1 [lindex $args 1]
            set p2 [lindex $args 2]
            set p3 [lindex $args 3]
            set p4 [lindex $args 4]
            set p5 [lindex $args 5]
            execute-command 16 "Enable Scan Table"  "3 $p1 $p2 $p3 $p4 $p5" 2
        }
        default     {puts "Unknown Acquisition Option [first $args]"}
    }   
}

proc do_GetOption {state args} {   

    do_State 

    return "none"
}

